import _pEvery from 'p-every';
import pLocate from 'p-locate';
import semver from 'semver';
import { CONDITIONS } from './plugin_conditions.js';
// the types of that package seem to be not correct and demand a `pEvery.default()` usage which is wrong
const pEvery = _pEvery;
/**
 *  Retrieve the `expectedVersion` of a plugin:
 *   - This is the version which should be run
 *   - This takes version pinning into account
 *   - If this does not match the currently cached version, it is installed first
 *  This is also used to retrieve the `compatibleVersion` of a plugin
 *   - This is the most recent version compatible with this site
 *   - This is the same logic except it does not use version pinning
 *   - This is only used to print a warning message when the `compatibleVersion`
 *     is older than the currently used version.
 */
export const getExpectedVersion = async function ({ versions, nodeVersion, packageJson, packagePath, buildDir, pinnedVersion, }) {
    const { version, conditions = [] } = await getCompatibleEntry({
        versions,
        nodeVersion,
        packageJson,
        packagePath,
        buildDir,
        pinnedVersion,
    });
    // Retrieve warning message shown when using an older version with `compatibility`
    const compatWarning = conditions.map(({ type, condition }) => CONDITIONS[type].warning(condition)).join(', ');
    return { version, compatWarning };
};
/**
 *  This function finds the right `compatibility` entry to use with the plugin.
 *   - `compatibility` entries are meant for backward compatibility
 *     Plugins should define each major version in `compatibility`.
 *   - The entries are sorted from most to least recent version.
 *   - After their first successful run, plugins are pinned by their major
 *     version which is passed as `pinnedVersion` to the next builds.
 *  When the plugin does not have a `pinnedVersion`, we use the most recent
 *  `compatibility` entry with a successful condition.
 *  When the plugin has a `pinnedVersion`, we do not use the `compatibility`
 *  conditions. Instead, we just use the most recent entry with a `version`
 *  matching `pinnedVersion`.
 *  When no `compatibility` entry matches, we use:
 *   - If there is a `pinnedVersion`, use it unless `latestVersion` matches it
 *   - Otherwise, use `latestVersion`
 */
const getCompatibleEntry = async function ({ versions, nodeVersion, packageJson, packagePath, buildDir, pinnedVersion, }) {
    const compatibleEntry = await pLocate(versions, async ({ version, overridePinnedVersion, conditions }) => {
        // Detect if the overridePinnedVersion intersects with the pinned version in this case we don't care about filtering
        const overridesPin = Boolean(pinnedVersion && overridePinnedVersion && semver.intersects(overridePinnedVersion, pinnedVersion));
        // ignore versions that don't satisfy the pinned version here if a pinned version is set
        if (!overridesPin && pinnedVersion && !semver.satisfies(version, pinnedVersion, { includePrerelease: true })) {
            return false;
        }
        // no conditions means nothing to filter
        if (conditions.length === 0 && pinnedVersion === undefined) {
            return false;
        }
        return await pEvery(conditions, async ({ type, condition }) => CONDITIONS[type].test(condition, { nodeVersion, packageJson, packagePath, buildDir }));
    });
    return (compatibleEntry ||
        (pinnedVersion ? { version: pinnedVersion, conditions: [] } : { version: versions[0].version, conditions: [] }));
};
